@@ -1,6 +1,6 @@
 #' Plot specimens in tangent space
 #'
-#' Function plots a set of Procrustes-aligned specimens in tangent space along their principal axes
+#' Function plots a set of Procrustes-aligned specimens in tangent space along their principal component axes
 #'
 #' The function performs a principal components analysis of shape variation and plots two 
 #'  dimensions of tangent space for a set of Procrustes-aligned specimens (default is PC1 vs. PC2). 
@@ -10,6 +10,9 @@
 #'  each group are plotted using distinct colors based on the order in which the groups are found in the dataset, 
 #'  and using R's standard color palette: black, red, green, blue, cyan, magenta, yellow, and gray. NOTE: to change
 #'  the colors of the groups, simply substitute a vector of the desired colors for each specimen (see example below).
+#'  
+#'  NOTE: previous versions of plotTangentSpace had option 'verbose' to return the PC scores and PC shapes. 
+#'  From version 3.0.2 this is automatic when assigned to an object.
 #'
 #' @param A An array (p x k x n) containing landmark coordinates for a set of aligned specimens 
 #' @param warpgrids A logical value indicating whether deformation grids for shapes along X-axis should be displayed
@@ -19,15 +22,15 @@
 #' @param axis2 A value indicating which PC axis should be displayed as the Y-axis (default = PC2)
 #' @param label An optional vector indicating labels for each specimen are to be displayed 
 #' (or if TRUE, numerical addresses are given)
-#' @param groups An optional factor vector specifying group identity for each specimen
-#' @param verbose A logical value indicating whether the output is basic or verbose (see Value below)
-#' @return Function returns a table summarizing the percent variation explained by each
-#'  pc axis (equivalent to summary() of \code{\link{prcomp}}) (default, verbose = FALSE).
-#'  If verbose=TRUE, function returns a list containing the following components: 
-#' \item{pc.summary}{A PC summary table as above}
-#' \item{pc.scores}{The set of principal component scores for all specimens.} 
-#' \item{pc.shapes}{A list with four components of the shape coordinates of the extreme ends of axis1 and axis2 
-#' is returned, which can be used by \code{\link{warpRefMesh}}.}
+#' @param groups An optional factor vector specifying group identity for each specimen (see example)
+#' @param legend A logical value for whether to add a legend to the plot (only when groups are assigned).
+#' @return If user assigns function to object, returned is a list of the following components:
+#' \item{pc.summary}{A table summarizing the percent variation explained by each pc axis, equivalent to summary of \code{\link{prcomp}}.}
+#' \item{pc.scores}{The set of principal component scores for all specimens.}
+#' \item{pc.shapes}{A list with the shape coordinates of the extreme ends of all PC axes, e.g. $PC1min}
+#' \item{sdev}{The standard deviations of the principal components (i.e., the square roots of the eigenvalues of the 
+#' covariance/correlation matrix, as per \code{\link{prcomp}}.}
+#' \item{rotation}{The matrix of variable loadings, as per \code{\link{prcomp}}.}
 #' @export
 #' @keywords visualization
 #' @author Dean Adams & Emma Sherratt
@@ -38,21 +41,24 @@
 #' gp <- as.factor(paste(plethodon$species, plethodon$site)) # group must be a factor
 #' plotTangentSpace(Y.gpa$coords, groups = gp) 
 #' 
-#' ## To plot residual shapes from an allometry regression (note: must add mean back in!) 
-#' plotTangentSpace(arrayspecs(resid(lm(two.d.array(Y.gpa$coords)~Y.gpa$Csize))+
-#'          predict(lm(two.d.array(Y.gpa$coords)~1)),12,2))
-#'  
+#' ## To save and use output
+#' PCA <- plotTangentSpace(Y.gpa$coords, groups = gp, legend=TRUE) 
+#' summary(PCA)
+#' PCA$pc.shapes
+#' PCA$rotation
+#' 
 #' ##To change colors of groups
 #' col.gp <- rainbow(length(levels(gp))) 
 #'    names(col.gp) <- levels(gp)
-#' col.gp <- col.gp[match(gp, names(col.gp))] # col.gp must not be a factor
-#' 
+#' col.gp <- col.gp[match(gp, names(col.gp))] # col.gp must NOT be a factor
 #' plotTangentSpace(Y.gpa$coords, groups = col.gp)
 #' 
-#' ## To make a quick legend [not run]
-#' #plot.new(); legend(0,1, legend= levels(gp), pch=19, col = levels(as.factor(col.gp)))
+#' ## To plot residual shapes from an allometry regression (note: must add mean back in!) 
+#' plotTangentSpace(arrayspecs(resid(lm(two.d.array(Y.gpa$coords)~Y.gpa$Csize))+
+#'          predict(lm(two.d.array(Y.gpa$coords)~1)),12,2))
+
 plotTangentSpace<-function (A, axis1 = 1, axis2 = 2, warpgrids = TRUE, mesh = NULL, label = NULL, 
-                            groups=NULL, verbose=FALSE){
+                            groups=NULL, legend=FALSE){
   if (length(dim(A)) != 3) {
     stop("Data matrix not a 3D array (see 'arrayspecs').") }
   if(any(is.na(A))==T){
@@ -65,6 +71,7 @@ plotTangentSpace<-function (A, axis1 = 1, axis2 = 2, warpgrids = TRUE, mesh = NU
   pc.res <- prcomp(x)
   pcdata <- pc.res$x
   if (warpgrids == FALSE) {
+    if(legend==TRUE){ layout(t(matrix(c(1, 1, 2, 1, 1, 1, 1, 1, 1), 3,3))) }
     plot(pcdata[, axis1], pcdata[, axis2], asp = 1, pch = 21,bg = "black", cex = 2, xlab = paste("PC ",axis1),
          ylab = paste("PC ",axis2))
     if(!is.null(groups)){points(pcdata[, axis1], pcdata[, axis2],pch=21,bg=groups,cex=2)}
@@ -74,31 +81,28 @@ plotTangentSpace<-function (A, axis1 = 1, axis2 = 2, warpgrids = TRUE, mesh = NU
       if(isTRUE(label)){text(pcdata[, axis1], pcdata[, axis2], seq(1, n), adj = c(-0.7, -0.7)) }
       else{text(pcdata[, axis1], pcdata[, axis2], label, adj = c(-0.1, -0.7)) }
     }
+    if(!is.null(groups) && legend==TRUE){
+        plot.new(); 
+        if(is.factor(groups)){legend(0.5,1, legend=unique(groups), pch=19, bty="n", col=unique(groups))
+        } else {legend(0.5,1, legend=unique(names(groups)), pch=19, bty="n", col=unique(groups)) }
+    }
   }
-  pcaxis.min.1 <- min(pcdata[, axis1])
-  pcaxis.max.1 <- max(pcdata[, axis1])
-  pc.min.1 <- pc.max.1 <- rep(0, dim(pcdata)[2])
-  pc.min.1[axis1] <- pcaxis.min.1
-  pc.max.1[axis1] <- pcaxis.max.1
-  shape.min.1 <- arrayspecs(as.matrix(pc.min.1 %*% (t(pc.res$rotation))), 
-                            p, k)[, , 1] + ref
-  shape.max.1 <- arrayspecs(as.matrix(pc.max.1 %*% (t(pc.res$rotation))), 
-                            p, k)[, , 1] + ref
-  pcaxis.min.2 <- min(pcdata[, axis2])
-  pcaxis.max.2 <- max(pcdata[, axis2])
-  pc.min.2 <- pc.max.2 <- rep(0, dim(pcdata)[2])
-  pc.min.2[axis2] <- pcaxis.min.2
-  pc.max.2[axis2] <- pcaxis.max.2
-  shape.min.2 <- arrayspecs(as.matrix(pc.min.2 %*% (t(pc.res$rotation))), 
-                            p, k)[, , 1] + ref
-  shape.max.2 <- arrayspecs(as.matrix(pc.max.2 %*% (t(pc.res$rotation))), 
-                            p, k)[, , 1] + ref
-  shapes <- list(shape.min.1, shape.max.1, shape.min.2, shape.max.2)
-  names(shapes) <- c(paste("PC",axis1,"min", sep=""),paste("PC",axis1,"max", sep=""),
-                     paste("PC",axis2,"min", sep=""),paste("PC",axis2,"max", sep=""))
+  shapes <- shape.names <- NULL
+  for(i in 1:ncol(pcdata)){
+    pcaxis.min <- min(pcdata[, i]) ; pcaxis.max <- max(pcdata[, i])
+    pc.min <- pc.max <- rep(0, dim(pcdata)[2])
+    pc.min[i] <- pcaxis.min ; pc.max[i] <- pcaxis.max
+    pc.min <- as.matrix(pc.min %*% (t(pc.res$rotation))) + as.vector(t(ref))
+    pc.max <- as.matrix(pc.max %*% (t(pc.res$rotation))) + as.vector(t(ref))
+    shapes <- rbind(shapes,pc.min, pc.max)
+    shape.names <- c(shape.names,paste("PC",i,"min", sep=""),paste("PC",i,"max", sep=""))
+  }
+  shapes <- arrayspecs(shapes,p, k)
+  shapes <- lapply(seq(dim(shapes)[3]), function(x) shapes[,,x])
+  names(shapes) <- shape.names
   if (warpgrids == TRUE) {
     if (k == 2) {
-      layout(t(matrix(c(2, 1, 1, 1, 1, 1, 1, 1, 3), 3,3)))
+      layout(t(matrix(c(2, 1, 4, 1, 1, 1, 1, 1, 3), 3,3)))
     }
     plot(pcdata[, axis1], pcdata[, axis2], asp = 1, pch = 21,bg = "black", cex = 2, xlab = paste("PC ",axis1),
          ylab = paste("PC ",axis2))
@@ -109,29 +113,40 @@ plotTangentSpace<-function (A, axis1 = 1, axis2 = 2, warpgrids = TRUE, mesh = NU
       if(isTRUE(label)){text(pcdata[, axis1], pcdata[, axis2], seq(1, n), adj = c(-0.7, -0.7)) }
       else{text(pcdata[, axis1], pcdata[, axis2], label, adj = c(-0.1, -0.1)) }
     }
+    shape.min <- shapes[[which(names(shapes) == paste("PC",axis1,"min", sep=""))]]
+    shape.max <- shapes[[which(names(shapes) == paste("PC",axis1,"max", sep=""))]]
     if (k == 2) {
       arrows(min(pcdata[, axis1]), (0.7 * max(pcdata[,axis2])), min(pcdata[, axis1]), 0, length = 0.1,lwd = 2)
       arrows(max(pcdata[, axis1]), (0.7 * min(pcdata[,axis2])), max(pcdata[, axis1]), 0, length = 0.1,lwd = 2)
-      tps(ref, shape.min.1, 20)
-      tps(ref, shape.max.1, 20)
+      tps(ref, shape.min, 20)
+      tps(ref, shape.max, 20)
     }
+    if(!is.null(groups) && legend==TRUE){
+      plot.new(); 
+      if(is.factor(groups)){legend(0.5,1, legend=unique(groups), pch=19, bty="n", col=unique(groups))
+        } else {legend(0.5,1, legend=unique(names(groups)), pch=19, bty="n", col=unique(groups)) }
+      }
     if (k == 3) {
       if (is.null(mesh)==TRUE){
-        open3d()
-        plot3d(shape.min.1, type = "s", col = "gray", main = paste("PC ", axis1," negative"),size = 1.25, aspect = FALSE)
-        open3d()
-        plot3d(shape.max.1, type = "s", col = "gray", main = paste("PC ", axis1," positive"), size = 1.25, aspect = FALSE)
+        open3d() ; mfrow3d(1, 2) 
+        plot3d(shape.min, type = "s", col = "gray", main = paste("PC ", axis1," negative"),size = 1.25, aspect = FALSE,xlab="",ylab="",zlab="",box=FALSE, axes=FALSE)
+        plot3d(shape.max, type = "s", col = "gray", main = paste("PC ", axis1," positive"), size = 1.25, aspect = FALSE,xlab="",ylab="",zlab="",box=FALSE, axes=FALSE)
         }
       if(is.null(mesh)==FALSE){
-        print("Warping mesh to axis 1 minima and maxima...")
-        plotRefToTarget(ref, shape.min.1, mesh, method = "surface")
+        open3d() ; mfrow3d(1, 2) 
+        cat(paste("\nWarping mesh to negative end of axis ", axis1, "\n", sep=""))
+        plotRefToTarget(ref, shape.min, mesh, method = "surface")
         title3d(main=paste("PC ", axis1," negative"))
-        plotRefToTarget(ref, shape.max.1, mesh, method = "surface")
+        next3d()
+        cat(paste("\nWarping mesh to positive end of axis ", axis1, "\n", sep=""))
+        plotRefToTarget(ref, shape.max, mesh, method = "surface")
         title3d(main=paste("PC ", axis1," positive"))
         }
       }
     layout(1)
     }
-  if(verbose==TRUE){ return(list(pc.summary = summary(pc.res), pc.scores = pcdata, pc.shapes= shapes)) }
-  if(verbose==FALSE){ return(pc.summary = summary(pc.res)) }
-}
\ No newline at end of file
+  out <- list(pc.summary = summary(pc.res), pc.scores = pcdata, pc.shapes = shapes, 
+              sdev = pc.res$sdev, rotation = pc.res$rotation)
+  class(out) = "plotTangentSpace"
+  out
+}
